//
// Copyright 2014 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.pdfextractor;

import java.io.IOException;
import java.io.InputStream;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.pdfbox.io.IOUtils;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;

import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 * 
 */
@SuppressWarnings({ "PMD.GodClass", "nls" })
public abstract class AbstractPdfExtractor {

	/**
	 * Threshold PDFBox value for keeping one word in one line.
	 */
	private static final int MAX_ONE_WORD = 1;
	/**
	 * Threshold PDFBox value for keeping 2 words in one line.
	 */
	protected static final int MAX_TWO_WORD = 2;
	/**
	 * The delimiter in the return string from the PDFBox extraction separating
	 * the positions and description values.
	 */
	protected static final String WORT_DELIMITER = "#\\*#"; //$NON-NLS-1$
	/**
	 * Ein extrahiertes Wort muss genau 3 Felder haben: PosX, PosY und das
	 * eigentliche Wort.
	 */
	protected static final int WORT_INFO_LENGTH = 3;
	/**
	 * Date of the bank statement
	 */
	protected transient String rechungsDatum;
	/**
	 * List of effective dates for entries
	 */
	protected transient List<PdfEintrag> buchungsDaten;
	/**
	 * List of acquisition dates for entries
	 */
	private transient List<PdfEintrag> kaufDaten;
	/**
	 * List of entry description texts
	 */
	private transient List<PdfEintrag> beschreibungen;
	/**
	 * List of amounts for entries
	 */
	private transient List<PdfEintrag> betraege;
	/**
	 * List with foreign currency values
	 */
	private transient List<PdfEintrag> fcValues;
	/**
	 * List with foreign currencys
	 */
	private transient List<PdfEintrag> foreignCurrencys;
	/**
	 * List with exchange rates
	 */
	private transient List<PdfEintrag> exchangeRates;

	/**
	 * Dezimalformatierer des Saldos
	 */
	private final transient NumberFormat numFormat;

	/**
	 * 
	 */
	public AbstractPdfExtractor() {

		this.numFormat = this.getNumberFormat();

	}

	/**
	 * Extracts all entries from the PDF bank statement and returns them in as a
	 * PdfBuchung list.
	 * 
	 * @param inputStream
	 *            of the bank statement PDF.
	 * @param kartenNummer
	 *            the credit card number to verify against the bank statement.
	 * @return a List of PdfBuchung with all found entries.
	 * @throws InvalidCardNumber
	 */
	public List<PdfBuchung> getExtract(final InputStream inputStream, final String kartenNummer)
			throws InvalidCardNumber {

		/**
		 * The credit card number for which the
		 */
		String kartenKonto;

		Double saldo = null;

		final List<PdfBuchung> buchungen = new ArrayList<>();

		try {

			final byte[] byteArray = IOUtils.toByteArray(inputStream);
			final PDDocument document = PDDocument.load(byteArray);
			Logger.debug("Seiten: " + document.getPages().getCount());

			final AbstractBankPDFPage page1 = this.getBankPdfFirstPage();
			final AbstractBankPDFPage evenPage = this.getBankPdfEvenPage();
			final AbstractBankPDFPage oddPage = this.getBankPdfOddPage();

			for (int pageNumber = 0; pageNumber < document.getPages().getCount(); pageNumber++) {
				final PDPage page = document.getPage(pageNumber);
				Logger.debug("Bearbeite Seite: " + pageNumber);
				try {
					AbstractBankPDFPage textAreas = null;
					if (pageNumber == 0) {
						textAreas = page1;
						textAreas.extractRegions(page);

						this.rechungsDatum = this.getRechungsDatum(textAreas);
						Logger.debug("Rechungsdatum: >" + this.rechungsDatum + "<");

						kartenKonto = this.getKartenkonto(textAreas);
						Logger.debug("Kartenkonto: >" + kartenKonto + "<");
						if (!kartenNummer.equals(kartenKonto)) {
							throw new InvalidCardNumber("Kartennumern stimmt nicht.");
						}

						saldo = this.getSaldo(textAreas);
						Logger.debug("Saldo: >" + saldo + "<");
					} else if (pageNumber % 2 == 1) { // NOPMD
						textAreas = evenPage;
						textAreas.extractRegions(page);
					} else {
						textAreas = oddPage;
						textAreas.extractRegions(page);
					}

					this.buchungsDaten = this.getBuchungsDaten(textAreas);
					this.logPdfWortList("Buchungsdatum", this.buchungsDaten);

					if (!this.buchungsDaten.isEmpty()) {
						this.kaufDaten = this.getKaufDaten(textAreas);
						this.logPdfWortList("KaufDatum", this.kaufDaten);

						this.beschreibungen = this.getBeschreibungen(textAreas);
						this.logPdfWortList("Beschreibung", this.beschreibungen);

						this.fcValues = this.getFremdwaehrungsBetraege(textAreas);
						this.logPdfWortList("Fremdwhrungsbetrge", this.fcValues);

						this.foreignCurrencys = this.getFremdwaehrungen(textAreas);
						this.logPdfWortList("Fremdwhrungen", this.foreignCurrencys);

						this.exchangeRates = this.getKurse(textAreas);
						this.logPdfWortList("Kurse", this.exchangeRates);

						this.betraege = this.getBetraege(textAreas);
						this.logPdfWortList("Betrge", this.betraege);

						final AbstractTransactionCreator taCreator = this.getTransactionCreator(); // NOPMD
						taCreator.addKaufDaten(this.kaufDaten);
						taCreator.addDescriptions(this.beschreibungen);
						taCreator.addFcValues(this.fcValues);
						taCreator.addForeignCurrencys(this.foreignCurrencys);
						taCreator.addExchangeRates(this.exchangeRates);
						taCreator.addBetraege(this.betraege);
						buchungen.addAll(taCreator.getTansactions());

					}
				} catch (IOException e) {
					Logger.warn("IO-Fehler beim extrahieren der Daten aus dem PDF. Fehler: " + e.getMessage());
					Logger.warn(e.getStackTrace().toString());
				}
			}
			document.close();
		} catch (IOException e) {
			Logger.warn(e.getMessage());
		}

		// Salden berechnen und Liste umdrehen

		this.addSaldoAndReorder(buchungen, saldo);
		return buchungen;

	}

	/**
	 * Soll die Umsatzliste umgedreht werden. Kann von Client-Implementationen
	 * &uuml;berschrieben werden. Default ist <i>true</i>.
	 * 
	 * @return <i>true</i>, falls die Umsatz-Liste umgedreht werden soll, sonst
	 *         <i>false</i>
	 */
	protected boolean isReOrder() {
		return true;
	}

	/**
	 * Liefert den konkreten TransactionCreator der Bank. Muss von der
	 * Unterklasse implementiert werden.
	 * 
	 * @return der konkreten TransactionCreator der Bank
	 */
	protected abstract AbstractTransactionCreator getTransactionCreator();

	/**
	 * Liefert das NumberFormat zum Parsen des Saldos. Muss von der Unterklasse
	 * implementiert werden.
	 * 
	 * @return das NumberFormat zum Parsen des Saldos
	 */
	protected abstract NumberFormat getNumberFormat();

	/**
	 * Liefert die PDFPage der ersten Seite des Auszugs. Muss von der
	 * Unterklasse implementiert werden.
	 * 
	 * @return die AbstractBankPDFPage der ersten Seite des Auszugs
	 * @throws IOException
	 */
	protected abstract AbstractBankPDFPage getBankPdfFirstPage() throws IOException;

	/**
	 * Liefert die PDFPage der geraden Seiten des Auszugs. Muss von der
	 * Unterklasse implementiert werden.
	 * 
	 * @return die AbstractBankPDFPage der geraden Seiten des Auszugs
	 * @throws IOException
	 */
	protected abstract AbstractBankPDFPage getBankPdfEvenPage() throws IOException;

	/**
	 * Liefert die PDFPage der ungeraden Seiten des Auszugs. Muss von der
	 * Unterklasse implementiert werden.
	 * 
	 * @return die AbstractBankPDFPage der ungeraden Seiten des Auszugs
	 * @throws IOException
	 */
	protected abstract AbstractBankPDFPage getBankPdfOddPage() throws IOException;

	/**
	 * Hier kann eine konkrete Implementierung noch &Auml;nderungen an der Liste
	 * der Buchungsdaten vornehmen. Zum Beispiel Kommentare/Werbung entfernen.
	 * 
	 * @param buchungenDatum
	 */
	protected abstract void removeAnnotationsBookingDates(List<PdfEintrag> buchungenDatum);

	/**
	 * Hier kann eine konkrete Implementierung noch &Auml;nderungen an der
	 * anderen Listen vornehmen. Zum Beispiel Kommentare/Werbung entfernen.
	 * 
	 * @param eintragsListe
	 */
	protected abstract void removeAnnotations(List<PdfEintrag> eintragsListe);

	/**
	 * Adds the year to each entry as this is might be not enclosed in the PDF
	 * bank statement entries.
	 * 
	 * @param entries
	 */
	protected abstract void addYear(final List<PdfEintrag> entries);

	/**
	 * Log on DEBUG level all words of a List<PdfEintrag>
	 * 
	 * @param prefix
	 * @param pdfWortList
	 * 
	 */
	private void logPdfWortList(final String prefix, final List<PdfEintrag> pdfWortList) {
		if (Logger.getLevel() == Level.DEBUG) {
			int zeile = 1;
			for (final PdfEintrag buchung : pdfWortList) {
				Logger.debug(prefix + "[" + zeile++ + "]: " + buchung.getWort() + ", X: " + buchung.getPositionX()
						+ ", Y: " + buchung.getPositionY());
			}
		}
	}

	/**
	 * Fgt die Salden zu den Buchungen und kehrt die Reihenfolge um.
	 * 
	 * @param buchungen
	 *            die Liste mit den Buchungen
	 * @param saldo
	 *            das Startsaldo
	 */
	private void addSaldoAndReorder(final List<PdfBuchung> buchungen, final Double saldo) {

		// Reihenfolge umdrehen
		if (this.isReOrder()) {
			Collections.reverse(buchungen);
		}

		// Salden eintragen
		if (saldo == null) {
			for (final PdfBuchung pdfBuchung : buchungen) {
				pdfBuchung.setSaldo(0);
			}
		} else {
			double zwischenSaldo = saldo;
			for (final PdfBuchung pdfBuchung : buchungen) {
				pdfBuchung.setSaldo(zwischenSaldo);
				zwischenSaldo = Utils.roundNumber(zwischenSaldo - pdfBuchung.getBetrag(), 2);
			}
		}
	}

	private String getRechungsDatum(final AbstractBankPDFPage textAreas) {

		final String[] parsedText = textAreas.getTextForRegion(AbstractBankPDFPage.RECHNUNGS_DATUM).split(
				AbstractPdfExtractor.WORT_DELIMITER);
		Logger.trace("PT (rechnungsdatum): " + parsedText.length);
		String datum = parsedText[2].replace("\n", "");
		datum = datum.replace("\r", "");
		return datum;
	}

	private String getKartenkonto(final AbstractBankPDFPage textAreas) {

		final String[] parsedText = textAreas.getTextForRegion(AbstractBankPDFPage.KARTEN_KONTO).split(
				AbstractPdfExtractor.WORT_DELIMITER);
		Logger.trace("PT (kartenkonto): " + parsedText.length);
		String kto = parsedText[2].replace("\n", "");
		kto = kto.replace("\r", "");
		kto = kto.replace(" ", "");
		return kto;
	}

	private Double getSaldo(final AbstractBankPDFPage textAreas) {

		final String[] parsedText = textAreas.getTextForRegion(AbstractBankPDFPage.SALDO).split(
				AbstractPdfExtractor.WORT_DELIMITER);
		Logger.trace("PT (kartenkonto): " + parsedText.length);
		String saldoString = parsedText[2].replace("\n", "");
		saldoString = saldoString.replace("\r", "");
		saldoString = saldoString.replace(" ", "");

		Double saldo = null;
		try {
			saldo = this.numFormat.parse(saldoString).doubleValue();
			// Check auf positiven Saldo
			if (saldo > 0) {
				saldo = saldo * -1.0;
			}
		} catch (ParseException e) {
			Logger.warn("Saldo nicht gefunden. Fehler: " + e.getMessage());
		}
		return saldo;
	}

	private List<PdfEintrag> getBuchungsDaten(final AbstractBankPDFPage textAreas) {

		final List<PdfEintrag> buchungenDatum = new ArrayList<>();

		final String text = textAreas.getTextForRegion(AbstractBankPDFPage.BUCHUNGS_DATUM);

		if (StringUtils.isNotEmpty(text)) {
			final String[] lines = text.split("\n");

			for (final String line : lines) {
				Logger.trace("Line (buchungsdatum): " + line);

				final String[] woerter = line.split(" ");

				if (woerter.length == 0 || woerter.length > AbstractPdfExtractor.MAX_ONE_WORD) {
					// keine Text oder mehr als eine Wort --> keine
					// Buchungszeile
					break;
				}
				final String[] wortInfo = woerter[0].split(AbstractPdfExtractor.WORT_DELIMITER);
				if (wortInfo.length != AbstractPdfExtractor.WORT_INFO_LENGTH) {
					Logger.debug("Zeile hat falsche Lnge. >" + line + "<");
					continue;
				}
				// X-Position berprfen
				final double xPosExpected = textAreas.getXPos();
				final Double xPosDiff = Double.valueOf(wortInfo[0]) - xPosExpected;
				if (-1 < xPosDiff && xPosDiff < 1) {
					final PdfEintrag datum = new PdfEintrag(); // NOPMD
					datum.setPositionX(Double.valueOf(wortInfo[0]));
					datum.setPositionY(Double.valueOf(wortInfo[1]));
					datum.setWort(wortInfo[2].replace("\n", ""));
					datum.setWort(datum.getWort().replace("\r", ""));
					buchungenDatum.add(datum);
				}
			}

			// Anmerkungen entfernen
			this.removeAnnotationsBookingDates(buchungenDatum);

		}
		// Jahr hinzu tun
		this.addYear(buchungenDatum);

		return buchungenDatum;
	}

	private List<PdfEintrag> getKaufDaten(final AbstractBankPDFPage textAreas) {

		final List<PdfEintrag> kaufDatums = new ArrayList<>();

		final String[] lines = textAreas.getTextForRegion(AbstractBankPDFPage.KAUF_DATUM).split("\n");

		for (final String line : lines) {
			Logger.trace("Line (kaufdatum): " + line);

			final String[] woerter = line.split(" ");

			if (woerter.length > AbstractPdfExtractor.MAX_ONE_WORD) {
				break;
			}

			for (final String wort : woerter) {

				final String[] wortInfo = wort.split(AbstractPdfExtractor.WORT_DELIMITER);
				final PdfEintrag kaufDatum = new PdfEintrag(); // NOPMD
				kaufDatum.setPositionX(Double.valueOf(wortInfo[0]));
				kaufDatum.setPositionY(Double.valueOf(wortInfo[1]));
				kaufDatum.setWort(wortInfo[2].replace("\n", ""));
				kaufDatum.setWort(kaufDatum.getWort().replace("\r", ""));
				kaufDatums.add(kaufDatum);
			}

		}

		// Anmerkungen entfernen
		this.removeAnnotations(kaufDatums);

		// Jahr hinzu tun
		this.addYear(kaufDatums);

		return kaufDatums;
	}

	private List<PdfEintrag> getBeschreibungen(final AbstractBankPDFPage textAreas) {

		final List<PdfEintrag> beschreibungen = new ArrayList<>();

		final String regioText = textAreas.getTextForRegion(AbstractBankPDFPage.BESCHREIBUNG);
		final String lineSeparator = textAreas.getLineSeparator();
		final String[] lines = regioText.split(lineSeparator);

		for (final String line : lines) {
			Logger.trace("Line (beschreibungen): >" + line + "<");

			if (StringUtils.isNotEmpty(line)) {
				final String[] woerter = line.split(" ");
				final PdfEintrag pdfEintrag = new PdfEintrag(); // NOPMD
				pdfEintrag.setPositionX(0);
				for (final String wort : woerter) {
					final String[] wortInfo = wort.split(AbstractPdfExtractor.WORT_DELIMITER);
					if (wortInfo.length != AbstractPdfExtractor.WORT_INFO_LENGTH) {
						continue;
					}
					if (pdfEintrag.getPositionX() == 0) {
						pdfEintrag.setPositionX(Double.valueOf(wortInfo[0]));
						pdfEintrag.setPositionY(Double.valueOf(wortInfo[1]));
						String wortText = wortInfo[2].replace("\n", "");
						wortText = wortText.replace("\r", "");
						pdfEintrag.setWort(wortText);
					} else {
						String wortText = wortInfo[2].replace("\n", "");
						wortText = wortText.replace("\r", "");
						pdfEintrag.setWort(pdfEintrag.getWort() + " " + wortText);
					}
				}
				beschreibungen.add(pdfEintrag);
			} else {
				final PdfEintrag pdfEintrag = new PdfEintrag(); // NOPMD
				pdfEintrag.setPositionX(191.34);
				pdfEintrag.setPositionY(0);
				pdfEintrag.setWort("");
				beschreibungen.add(pdfEintrag);
			}

		}

		// Anmerkungen entfernen
		this.removeAnnotations(beschreibungen);

		return beschreibungen;
	}

	private List<PdfEintrag> getFremdwaehrungsBetraege(final AbstractBankPDFPage textAreas) {

		final List<PdfEintrag> fwBetraege = new ArrayList<>();

		String regioText = textAreas.getTextForRegion(AbstractBankPDFPage.FREMDW_BETRAG);
		final String lineSeparator = textAreas.getLineSeparator();

		if (lineSeparator.equals(regioText)) {
			regioText = StringUtils.EMPTY;
		}

		final String[] lines = regioText.split(lineSeparator);

		for (final String line : lines) {
			Logger.trace("Line (fwBetrag): >" + line + "<");

			if (StringUtils.isNotEmpty(line)) {
				final String[] woerter = line.split(" ");

				if (woerter.length > AbstractPdfExtractor.MAX_ONE_WORD) {
					break;
				}
				final PdfEintrag pdfEintrag = new PdfEintrag(); // NOPMD
				if (woerter.length == 0) {
					pdfEintrag.setWort("");
				} else {
					final String wort = woerter[0];
					final String[] wortInfo = wort.split(AbstractPdfExtractor.WORT_DELIMITER);
					pdfEintrag.setPositionX(Double.valueOf(wortInfo[0]));
					pdfEintrag.setPositionY(Double.valueOf(wortInfo[1]));
					pdfEintrag.setWort(wortInfo[2].replace("\n", ""));
					pdfEintrag.setWort(pdfEintrag.getWort().replace("\r", ""));
				}
				fwBetraege.add(pdfEintrag);
			}
		}

		// Anmerkungen entfernen
		this.removeAnnotations(fwBetraege);

		return fwBetraege;
	}

	private List<PdfEintrag> getFremdwaehrungen(final AbstractBankPDFPage textAreas) {

		final List<PdfEintrag> fremdwaehrungen = new ArrayList<>();
		final String lineSeparator = textAreas.getLineSeparator();

		String regioText = textAreas.getTextForRegion(AbstractBankPDFPage.FREMDWAEHRUNG);
		if (lineSeparator.equals(regioText)) {
			regioText = StringUtils.EMPTY;
		}
		final String[] lines = regioText.split(lineSeparator);

		for (final String line : lines) {
			Logger.trace("Line (fremdwhrung): " + line);

			final String[] woerter = line.split(" ");

			if (woerter.length > AbstractPdfExtractor.MAX_ONE_WORD) {
				break;
			}
			final PdfEintrag pdfEintrag = new PdfEintrag(); // NOPMD
			if (woerter.length == 0) {
				pdfEintrag.setWort("");
			} else {
				final String wort = woerter[0];
				final String[] wortInfo = wort.split(AbstractPdfExtractor.WORT_DELIMITER);
				if (wortInfo.length == AbstractPdfExtractor.WORT_INFO_LENGTH) {
					pdfEintrag.setPositionX(Double.valueOf(wortInfo[0]));
					pdfEintrag.setPositionY(Double.valueOf(wortInfo[1]));
					pdfEintrag.setWort(wortInfo[2].replace("\n", ""));
					pdfEintrag.setWort(pdfEintrag.getWort().replace("\r", ""));
				} else {
					pdfEintrag.setWort("");
				}
			}

			fremdwaehrungen.add(pdfEintrag);

		}

		// Anmerkungen entfernen
		this.removeAnnotations(fremdwaehrungen);

		return fremdwaehrungen;
	}

	private List<PdfEintrag> getKurse(final AbstractBankPDFPage textAreas) {

		final List<PdfEintrag> kurse = new ArrayList<>();
		final String lineSeparator = textAreas.getLineSeparator();

		final String regioText = textAreas.getTextForRegion(AbstractBankPDFPage.KURS);
		final String[] lines = regioText.split(lineSeparator);

		for (final String line : lines) {
			Logger.trace("Line (fwBetrag): >" + line + "<");

			if (StringUtils.isNotEmpty(line)) {
				final String[] woerter = line.split(" ");

				if (woerter.length > AbstractPdfExtractor.MAX_ONE_WORD) {
					break;
				}
				final PdfEintrag pdfEintrag = new PdfEintrag(); // NOPMD

				if (woerter.length == 0) {
					pdfEintrag.setWort("");
				} else {
					final String wort = woerter[0];
					final String[] wortInfo = wort.split(AbstractPdfExtractor.WORT_DELIMITER);
					if (wortInfo.length == AbstractPdfExtractor.WORT_INFO_LENGTH) {
						pdfEintrag.setWort("");
						pdfEintrag.setPositionX(Double.valueOf(wortInfo[0]));
						pdfEintrag.setPositionY(Double.valueOf(wortInfo[1]));
						pdfEintrag.setWort(wortInfo[2].replace("\n", ""));
						pdfEintrag.setWort(pdfEintrag.getWort().replace("\r", ""));
					} else {
						pdfEintrag.setWort("");
					}
				}
				kurse.add(pdfEintrag);
			}
		}

		// Anmerkungen entfernen
		this.removeAnnotations(kurse);

		return kurse;
	}

	/**
	 * Liefert die Betr&auml;ge der Ums&auml; auf dem PDF
	 * 
	 * @param textAreas
	 *            die TextAre ader Betr&auml;ge.
	 * 
	 * @return Liefert die Betr&auml;ge
	 */
	protected List<PdfEintrag> getBetraege(final AbstractBankPDFPage textAreas) {

		final List<PdfEintrag> betraege = new ArrayList<>();
		final String lineSeparator = textAreas.getLineSeparator();

		final String regioText = textAreas.getTextForRegion(AbstractBankPDFPage.BETRAG);
		final String[] lines = regioText.split(lineSeparator);

		for (final String line : lines) {
			Logger.trace("Line (betrag): >" + line + "<");

			if (StringUtils.isNotEmpty(line)) {
				final String[] woerter = line.split(" ");

				if (woerter.length == 0 || woerter.length > AbstractPdfExtractor.MAX_ONE_WORD) {
					break;
				}
				final String wort = woerter[0];
				final String[] wortInfo = wort.split(AbstractPdfExtractor.WORT_DELIMITER);
				if (wortInfo.length != AbstractPdfExtractor.WORT_INFO_LENGTH) {
					continue;
				}
				final PdfEintrag pdfEintrag = new PdfEintrag(); // NOPMD
				pdfEintrag.setPositionX(Double.valueOf(wortInfo[0]));
				pdfEintrag.setPositionY(Double.valueOf(wortInfo[1]));
				pdfEintrag.setWort(wortInfo[2].replace("\n", ""));
				pdfEintrag.setWort(pdfEintrag.getWort().replace("\r", ""));
				betraege.add(pdfEintrag);
			}
		}

		// Anmerkungen entfernen
		this.removeAnnotations(betraege);

		return betraege;
	}

}