//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.pdfextractor;

import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.logging.Logger;

/**
 * Erstellt eine Liste mit Hibiscus-Umstzen.
 * 
 * @author Frank Mnster
 *
 */
public abstract class AbstractTransactionCreator implements InterfaceTransactionsCreator {

	/**
	 * Die List emit den Buchungen
	 */
	private final transient List<PdfBuchung> transactions;

	/**
	 * Liste mit den Positionen der einzelnen Zeilen
	 */
	private final transient Map<Long, Integer> taNumbers;

	/**
	 * Der Datumsformatierer
	 */
	protected final transient SimpleDateFormat sdf;

	/**
	 * Dezimalformatierer
	 */
	private final transient NumberFormat numFormat;

	/**
	 * Y-Position der ersten Buchung auf der Seite.
	 */
	private final transient double firstLineY;

	/**
	 * Die Zeilenhhe
	 */
	private final transient double lineHeigh;

	/**
	 * Konstruktor mit Angabe der Buchungsdatums-Liste. Daran werden dann alle
	 * anderen Zeilen anhand ihrer Hhe gemessen.
	 * 
	 * @param buchungsDaten
	 *            die Liste mit den Buchungsdaten
	 * @param lineHeigh
	 *            die Zeilenhhe. Wird bentigt, um die Buchungszeilnummer zu
	 *            berechnen.
	 */
	public AbstractTransactionCreator(final List<PdfEintrag> buchungsDaten, final double lineHeigh) {

		sdf = getDateFormat();
		numFormat = getNumberFormat();

		transactions = new ArrayList<>();
		taNumbers = new HashMap<Long, Integer>();

		this.lineHeigh = lineHeigh;
		firstLineY = buchungsDaten.get(0).getPositionY();

		Date bDatum = null;
		int taNumber = 0;
		for (final PdfEintrag buchungsDatum : buchungsDaten) {
			try {
				bDatum = sdf.parse(buchungsDatum.getWort());
				final PdfBuchung buchung = new PdfBuchung(); // NOPMD
				buchung.setBuchungsDatum(bDatum);
				transactions.add(buchung);
				final long buchungsZeile = Math.round((firstLineY - buchungsDatum.getPositionY()) / lineHeigh);
				taNumbers.put(buchungsZeile, taNumber++);
			} catch (ParseException e) {
				Logger.error(I18N.tr("AbstractTransactionCreator.bookingDateParseError", e.getMessage())); //$NON-NLS-1$
			}
		}

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.InterfaceTransactionsCreator#addKaufDaten(java.util.List)
	 */
	@Override
	public void addKaufDaten(final List<PdfEintrag> valutas) {

		Date vDatum = null;
		for (final PdfEintrag valutaDatum : valutas) {
			try {
				synchronized (AbstractTransactionCreator.class) {
					vDatum = sdf.parse(valutaDatum.getWort());
				}
				final Integer number = getRow(valutaDatum.getPositionY());
				if (number != null) {
					final PdfBuchung transaction = transactions.get(number);
					transaction.setKaufDatum(vDatum);
				}
			} catch (ParseException e) {
				Logger.error(I18N.tr("AbstractTransactionCreator.valutaParseError") + e.getMessage()); //$NON-NLS-1$
			}
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.InterfaceTransactionsCreator#addDescriptions(java.util.List)
	 */
	@Override
	public void addDescriptions(final List<PdfEintrag> describtions) {
		for (final PdfEintrag description : describtions) {
			final Integer row = getRow(description.getPositionY());
			if (row != null) {
				final PdfBuchung transaction = transactions.get(row);
				transaction.setZweck(description.getWort());
			}
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.InterfaceTransactionsCreator#addFcValues(java.util.List)
	 */
	@Override
	public void addFcValues(final List<PdfEintrag> fcValues) {
		Double fcValue;
		for (final PdfEintrag fcValueWord : fcValues) {
			try {
				fcValue = numFormat.parse(fcValueWord.getWort()).doubleValue();
				final Integer row = getRow(fcValueWord.getPositionY());
				if (row != null) {
					final PdfBuchung transaction = transactions.get(row);
					transaction.setFremdWaehrungsBetrag(fcValue);
				}
			} catch (ParseException e) {
				Logger.error(I18N.tr("AbstractTransactionCreator.foreignCurrencyParseError") + e.getMessage()); //$NON-NLS-1$
			}
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.InterfaceTransactionsCreator#addForeignCurrencys(java.util.List)
	 */
	@Override
	public void addForeignCurrencys(final List<PdfEintrag> foreignCurrencys) {

		for (final PdfEintrag foreignCurrency : foreignCurrencys) {
			final Integer row = getRow(foreignCurrency.getPositionY());
			if (row != null) {
				final PdfBuchung transaction = transactions.get(row);
				transaction.setFremdWaehrung(foreignCurrency.getWort());
			}
		}

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.InterfaceTransactionsCreator#addExchangeRates(java.util.List)
	 */
	@Override
	public void addExchangeRates(final List<PdfEintrag> exchangeRates) {
		Double rate;
		for (final PdfEintrag rateWord : exchangeRates) {
			try {
				rate = numFormat.parse(rateWord.getWort()).doubleValue();
				final Integer row = getRow(rateWord.getPositionY());
				if (row != null) {
					final PdfBuchung transaction = transactions.get(row);
					transaction.setKurs(rate);
				}
			} catch (ParseException e) {
				Logger.error(I18N.tr("AbstractTransactionCreator.exchangeRateParseError") + e.getMessage()); //$NON-NLS-1$
			}
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.InterfaceTransactionsCreator#addBetraege(java.util.List)
	 */
	@Override
	public void addBetraege(final List<PdfEintrag> values) {
		Double value = null;
		for (final PdfEintrag valuesWord : values) {
			try {
				value = numFormat.parse(valuesWord.getWort()).doubleValue();
				final Integer row = getRow(valuesWord.getPositionY());
				if (row != null) {
					final PdfBuchung transaction = transactions.get(row);
					transaction.setBetrag(value);
				}
			} catch (ParseException e) {
				Logger.error(I18N.tr("AbstractTransactionCreator.valueParseError") + e.getMessage()); //$NON-NLS-1$
			}
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.InterfaceTransactionsCreator#getTansactions()
	 */
	@Override
	public List<PdfBuchung> getTansactions() {
		return transactions;
	}

	/**
	 * Liefert das Datumsformat in dem die Daten verarbeitet werden sollen. Muss
	 * von der Unterklasse implementiert werden.
	 * 
	 * @return das Datumsformt der Kreditkartenausz&uuml;ge
	 */
	protected abstract SimpleDateFormat getDateFormat();

	/**
	 * Liefert das NumberFormat in dem die Betr&auml;ge verarbeitet werden
	 * sollen. Muss von der Unterklasse implementiert werden.
	 * 
	 * @return das Zahlenformt der Kreditkartenausz&uuml;ge
	 */
	protected abstract NumberFormat getNumberFormat();

	/**
	 * Liefert die Zeilenummer zu einer Y-Position
	 * 
	 * @param posY
	 *            die y-Position
	 * @return die Zeilenummer
	 */
	private Integer getRow(final double posY) {
		final long myRow = Math.round((firstLineY - posY) / lineHeigh);
		return taNumbers.get(myRow);
	}

}
